@@ -1,12 +1,13 @@
-from kart.utils import KartMap
-from kart.utils import StoppableThread
-
-from http.server import HTTPServer, SimpleHTTPRequestHandler
+import argparse
 import os
 import shutil
-import argparse
-import threading
 import sys
+import threading
+import traceback
+from copy import deepcopy
+from http.server import HTTPServer, SimpleHTTPRequestHandler
+
+from kart.utils import KartMap, KartObserver
 
 
 class Kart:
@@ -30,23 +31,25 @@ class Kart:
         self.lock = threading.Lock()
 
     def prepare(self):
-        self.site = {}
+        self.site = {"config": self.config}
         for miner in self.miners:
+            miner.read_data()
             self.site.update(miner.collect())
-        self.site["config"] = self.config
         for modifier in self.content_modifiers:
             modifier.modify(self.site)
 
     def create_map(self):
         self.map = KartMap(self.config["base_url"])
         for mapper in self.mappers:
-            self.map.update(mapper.map(self.site))
+            self.map.update(mapper.map(deepcopy(self.site)))
         for modifier in self.map_modifiers:
             modifier.modify(self.map, self.site)
 
     def write(self):
         for renderer in self.renderers:
-            renderer.render(self.map, self.site, self.build_location)
+            map = deepcopy(self.map)
+            site = deepcopy(self.site)
+            renderer.render(map, site, self.build_location)
 
     def build(self):
         self.prepare()
@@ -60,16 +63,35 @@ class Kart:
     # therefore it is not possible to access only partial data,
     # preventing errors when serving the site during development
 
+    def update_data(self):
+        self.site = {"config": self.config}
+        for miner in self.miners:
+            self.site.update(miner.collect())
+        for modifier in self.content_modifiers:
+            modifier.modify(self.site)
+        self.create_map()
+        _site = deepcopy(self.site)
+        _map = deepcopy(self.map)
+        _urls = {}
+        for slug, page in self.map.items():
+            _urls[page["url"]] = slug
+        with self.lock:
+            self._site = _site
+            self._map = _map
+            self._urls = _urls
+
     def get_url_data(self, url):
         with self.lock:
-            if url not in [x["url"] for x in self._map.values()]:
-                # TODO: generalize using regex
-                if url.split("/")[1] == "static":
-                    url = "/static/*"
-                else:
-                    url = "/*"
-            page = [x for x in self._map.values() if x["url"] == url][0]
-            return page
+            _map = self._map
+            _urls = self._urls
+        if url not in _urls.keys():
+            # TODO: generalize using regex
+            if url.split("/")[1] == "static":
+                url = "/static/*"
+            else:
+                url = "/*"
+        page = _map[_urls[url]]
+        return page
 
     def get_global_data(self):
         with self.lock:
@@ -79,51 +101,44 @@ class Kart:
         get_url_data = self.get_url_data
         get_global_data = self.get_global_data
         renderer_dict = {}
+        observer = KartObserver()
+        observer.action = self.update_data
+        for miner in self.miners:
+            miner.start_watching(observer)
+        observer.start()
         for renderer in self.renderers:
             renderer_dict[renderer.name] = renderer
             renderer.start_serving()
 
         class RequestHandler(SimpleHTTPRequestHandler):
-            def __init__(self, *args, **kwargs):
-                super().__init__(*args, **kwargs)
-
             def do_GET(self):
                 page = get_url_data(self.path)
                 map, site = get_global_data()
-                if not page:
-                    return
-                else:
+                if page:
                     try:
                         renderer_dict[page["renderer"]].serve(self, page, map, site)
-                    except Exception as e:
-                        print(e)
+                    except Exception:
+                        print(traceback.format_exc())
 
-        handler = RequestHandler
-        httpd = HTTPServer(("", port), handler)
+        httpd = HTTPServer(("", port), RequestHandler)
         httpd.timeout = 0.1
-        server_thread = StoppableThread(target=httpd.handle_request)
-        self.prepare()
-        self.create_map()
-        self._site = self.site
-        self._map = self.map
-        server_thread.start()
+        self.update_data()
+        shutil.rmtree(self.build_location, ignore_errors=True)
 
         while True:
             try:
-                shutil.rmtree(self.build_location, ignore_errors=True)
-                self.prepare()
-                self.create_map()
+                httpd.handle_request()
             except KeyboardInterrupt:
                 print("\rexiting")
-                server_thread.stop()
+                for miner in self.miners:
+                    miner.stop_watching()
                 for renderer in self.renderers:
                     renderer.stop_serving()
+                observer.stop()
+                observer.join()
                 sys.exit()
             except Exception:
                 pass
-            with self.lock:
-                self._site = self.site
-                self._map = self.map
 
     def run(self):
         parser = argparse.ArgumentParser()
