@@ -1,47 +1,20 @@
-import threading
-import mistune
-from pygments import highlight
-from pygments.lexers import get_lexer_by_name
-from pygments.formatters import HtmlFormatter
 import re
-from collections import UserDict, UserString
+from collections import UserDict
 from itertools import islice
 from math import ceil
 
+from watchdog.observers import Observer
 
-class StoppableThread(threading.Thread):
-    def __init__(self, target):
-        super().__init__()
-        self._stop_event = threading.Event()
-        self.target = target
-
-    def stop(self):
-        self._stop_event.set()
 
+class KartObserver(Observer):
     def run(self):
-        while not self._stop_event.is_set():
-            self.target()
-
-
-class KartMistuneRenderer(mistune.Renderer):
-    def block_code(self, text, lang):
-        if not lang:
-            text = text.strip()
-            return f"<pre><code>{mistune.escape(text)}</code></pre>\n"
-        try:
-            lexer = get_lexer_by_name(lang, stripall=True)
-            formatter = HtmlFormatter(wrapcode=True)
-            code = highlight(text, lexer, formatter)
-            return code
-        except:
-            return f'<pre class="{lang}"><code>{mistune.escape(text)}</code></pre>\n'
-
-    def header(self, text, level, raw=None):
-        slug = re.sub(r"[\W_]+", "-", text)
-        return f"<h{level} id={slug}>{text}</h{level}>\n"
-
-
-markdown = mistune.Markdown(renderer=KartMistuneRenderer())
+        while self.should_keep_running():
+            try:
+                self.dispatch_events(self.event_queue, self.timeout)
+                if self.event_queue.empty():
+                    self.action()
+            except Exception:
+                continue
 
 
 class KartMap(UserDict):
